/*
Copyright (c) 2004 TrashHalo

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

//DEFINING TOKENIZER CLASS
//The tokenizer class is used for splitting a string by
//a string a of characters. I like this better than a classic
//split because it only does it up to where you need it
//plus you dont have a giant array sitting around holding each
//token
//
//example:
//var/Tokenizer/t
//t=new("Hello|World","|")
//world << t.next()
//world << t.next()
//
//output:
//Hello
//World
var/Tokenizer
Tokenizer/var
	String
	Token
	Current=1
Tokenizer/New(s,t)
	String=s;
	Token=t;
Tokenizer/proc
//next() returns characters until the token is found
	next()
		var/tmpCurrent=0;
		var/subStr;
		if(Current == length(String))
			subStr = "";
		else
			tmpCurrent = findText(String,Token,Current);
			if(tmpCurrent == 0)
				subStr=copytext(String,Current,length(String)+1);
				Current=length(String);
			else
				subStr = copytext(String,Current,tmpCurrent);
				Current=tmpCurrent+length(Token);
		return subStr;
//more() reports if there are any more tokens left 1 if true
//0 if false
	more()
		var/returner=1;
		if(Current == length(String))
			returner=0
		return returner
//END CLASS DEFINITION

proc
//Trys to find a string at the begining of a string 1 if true
//0 if false
	startsWith(base as text,find as text)
		var/index
		var/returner=0
		index = findText(base,find);
		if(index==1)
			returner = 1;
		return returner;
//Trys to find a string at the end of a string 1 if true 0 if
//false
	endsWith(base as text,find as text)
		var/index
		var/returner=0
		index = findText(base,find,length(base)-length(find))
		if(index != 0)
			returner =1;
		return returner
//removes all the spaces from the start and end of a string
	trim(base as text)
		var/returner
		var/leftPadding=0
		var/rightPadding=0
		var/end=0
		var/index
		var/indexTab
		if((startsWith(base," "))||(startsWith(base,"\t")))
			while(end==0)
				index=findText(base," ",leftPadding+1,leftPadding+2);
				indexTab=findText(base,"\t",leftPadding+1,leftPadding+2);
				if((index!=0)||(indexTab!=0))
					leftPadding += 1;
				else
					end=1
		if((endsWith(base," "))||(endsWith(base,"\t")))
			end=0
			while(end==0)
				index=findText(base," ",length(base)-rightPadding,length(base)-rightPadding+1);
				indexTab=findText(base,"\t",length(base)-rightPadding,length(base)-rightPadding+1);
				if((index!=0)||(indexTab!=0))
					rightPadding += 1;
				else
					end=1
		returner = copytext(base,leftPadding+1,length(base)-rightPadding+1);
		return returner;
//returns the character that is at a specific index in a string
	charAt(base as text,at as num)
		return copytext(base,at,at+1)
//replaces all instances of a string with a alternative string
	replace(base as text,find as text,replace as text)
		var/returner
		var/Tokenizer/tokenStr
		tokenStr = new(base,find)
		while(tokenStr.more() != 0)
			returner += tokenStr.next() + replace;
		returner=copytext(returner,1,length(returner)+1-length(replace));
		return returner;
//removes the end character from a string, only if its a end-line
	chomp(base as text)
		if(charAt(base,length(base))=="\n")
			return copytext(base,1,length(base))
		else
			return base
//removes the end character from a string
	chop(base as text)
		return copytext(base,1,length(base))
//concatenates a array of strings placing a token string between each element
//of the array
	join(token as text,base[])
		var/returner
		for(var/i=1,i<=base.len,i++)
			if(i==base.len)
				returner += base[i]
			else
				returner += base[i]+token
		return returner
//splits a string into a array of strings seperating them by the specified token.
//This is like the tokenizer but it does it all at once.
	split(token as text,base as text)
		var/list/returner
		returner = new()
		var/Tokenizer/splitter
		splitter=new(base,token)
		while(splitter.more() != 0)
			returner.Add(splitter.next())
		return returner
//seperates a string into a array of characters.
	toCharArray(base as text)
		var/list/returner
		returner=new()
		for(var/i=1,i<=length(base),i++)
			returner.Add(charAt(base,i))
		return returner
//wraps text to the number of chars you specifiy. Pretty efficient function, during
//tests it could process 40000 characters in a average of 5miliseconds
	wordWrap(base as text,numChars as num)
		var/Tokenizer/splitByLine
		var/Tokenizer/splitBySpace
		var/temp = ""
		var/line = ""
		var/returner = ""
		splitByLine = new(base,"\n")
		while(splitByLine.more() != 0)
			splitBySpace = new(splitByLine.next()," ")
			while(splitBySpace.more() !=0)
				temp=splitBySpace.next()
				if(length(line+temp)<=numChars)
					line += temp + " "
					if(splitBySpace.more() == 0)
						returner += line
						line = ""
				else
					returner += line + "\n"
					line = temp + " "
			returner = addtext(returner,line)
			returner = addtext(returner,"\n")
			line=""
		return chomp(returner)
//counts the number of times a specific pattern exists in a string
	count(base as text,token as text)
		var/total=0
		var/current=1
		var/findt
		do
			findt=findText(base,token,current)
			if(findt != 0)
				current=findt+length(token)
				total++
		while(findt != 0)
		return total
//reverses the characters in a string. I am working on getting this one
//as optimized as possible but currently I cant reccomend running it on
//a string bigger than 10000 chars. it will take about 10 mili to
//complete that.
	reverse(base as text)
		var/returner
		var/len=length(base)
		var/i
		for(i=0,i<len,i++)
			returner += copytext(base,len-i,len+1-i)
		return returner
